@@ -1,10 +1,12 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 import argparse
+import math
 import re
 import shlex
 import sys
-from typing import List, Tuple
+from typing import List, Tuple, Dict
+from collections import OrderedDict
 
 import configargparse
 
@@ -21,7 +23,7 @@ except ModuleNotFoundError:
     from selector import Selector
     from paths import *
 
-__version__ = '5.2.0'
+__version__ = '5.3.0'
 
 
 class Rofimoji:
@@ -121,6 +123,13 @@ class Rofimoji:
             default=10,
             help='Show at most this number of recently used characters (cannot be larger than 10)'
         )
+        parser.add_argument(
+            '--no-frecency',
+            dest='frecency',
+            action='store_false',
+            help='Don\'t show frequently used characters first'
+        )
+        parser.set_defaults(frecency=True)
         parser.add_argument(
             '--selector',
             dest='selector',
@@ -235,6 +244,7 @@ class Rofimoji:
                 self.choose_action_from_return_code(returncode)
                 characters = self.process_chosen_characters(stdout.splitlines())
 
+            self.save_characters_to_recent_file(characters)
             self.execute_action(characters)
 
     def mode_show_characters(self) -> None:
@@ -248,10 +258,11 @@ class Rofimoji:
     def mode_act_on_selection(self, chosen_character: str, returncode: int) -> None:
         if 10 <= returncode <= 19:
             characters = self.load_recent_characters(self.args.max_recent)[returncode - 10].strip()
+            self.save_characters_to_recent_file(characters)
             self.execute_action(characters)
         else:
             self.choose_action_from_return_code(returncode)
-            self.save_selection_to_cache(re.match(r'^(?:\u200e(?! ))?(?P<char>.[^ ]*) .*', chosen_character).group('char'), '')
+            self.save_selection_to_cache( self._parse_line(chosen_character), '')
             self.mode_select_skin_tone('')
 
     def mode_select_skin_tone(self, processed_character: str) -> None:
@@ -259,6 +270,7 @@ class Rofimoji:
         processed_characters += processed_character.split(' ')[0]
 
         for character in characters:
+            self.save_characters_to_frecency_file(characters.split(' ')[0])
             if character not in self.skin_tone_selectable_emojis:
                 processed_characters += character
                 characters = characters[1:]
@@ -286,17 +298,33 @@ class Rofimoji:
         elif return_code == 24:
             self.args.action = Action.COPY_UNICODE
 
+    def _parse_line(self, line) -> str:
+        return re.match(r'^(?:\u200e(?! ))?(?P<char>.[^ ]*) .*', line).group('char')
+
     def read_character_files(self) -> str:
-        return ''.join(self.load_from_file(file_name) for file_name in self.resolve_all_files())
+        all_characters = OrderedDict()
+
+        if self.args.frecency:
+            for character in self.read_frecencies().keys():
+                all_characters[character] = ''
+
+        for file_name in self.resolve_all_files():
+            characters_from_file = self.load_from_file(file_name)
+            for line in characters_from_file:
+                parsed_line = line.split(' ', 1)
+                all_characters[parsed_line[0]] = all_characters.get(parsed_line[0], '') + parsed_line[1] if 1 < len(parsed_line) else ''
+
+        return '\n'.join(f"{key} {value}" for key, value in all_characters.items() if value != '')
 
     def resolve_all_files(self) -> List[str]:
         file_names = self.args.files
 
         if len(file_names) == 1 and file_names[0] == 'all':
             file_names = [file.stem for file in (Path(__file__).parent / "data").glob("*.csv")]
+
         return file_names
 
-    def load_from_file(self, file_name: str) -> str:
+    def load_from_file(self, file_name: str) -> List[str]:
         provided_file = Path(__file__).parent / "data" / f"{file_name}.csv"
         if Path(file_name).expanduser().is_file():
             actual_file_name = Path(file_name).expanduser()
@@ -305,7 +333,7 @@ class Rofimoji:
         else:
             raise FileNotFoundError(f"Couldn't find file {file_name!r}")
 
-        return actual_file_name.read_text()
+        return actual_file_name.read_text().strip().split('\n')
 
     def load_recent_characters(self, max: int) -> List[str]:
         try:
@@ -329,11 +357,13 @@ class Rofimoji:
         )
 
     def process_chosen_characters(self, chosen_characters: List[str]) -> str:
+        for character in chosen_characters:
+            self.save_characters_to_frecency_file(character.split(' ')[0])
+
         processed_characters = ''.join(
-            self.add_skin_tone(re.match(r'^(?:\u200e(?! ))?(?P<char>.[^ ]*) .*', character).group('char'))
+            self.add_skin_tone(self._parse_line(character))
             for character in chosen_characters
         )
-        self.save_characters_to_recent_file(processed_characters)
 
         return processed_characters
 
@@ -382,7 +412,7 @@ class Rofimoji:
             return
 
         old_file_name = recents_file_location
-        new_file_name = old_file_name.with_name('recent_temp')
+        new_file_name = old_file_name.with_name('recent.tmp')
 
         max_recent = min(max_recent_from_conf, 10)
 
@@ -406,6 +436,36 @@ class Rofimoji:
 
         new_file_name.rename(old_file_name)
 
+    def read_frecencies(self) -> Dict[str, int]:
+        frecencies = {}
+        try:
+            with frecency_file_location.open('r') as file:
+                for line in file:
+                    (frecency, character) = line.strip().split(' ')
+                    frecencies[character] = int(frecency)
+        except FileNotFoundError:
+            pass
+
+        return frecencies
+
+    def save_characters_to_frecency_file(self, chosen_character: str) -> None:
+        if not self.args.frecency:
+            return
+
+        new_file_name = frecency_file_location.with_name('frecency.tmp')
+
+        new_file_name.parent.mkdir(parents=True, exist_ok=True)
+
+        frecencies = self.read_frecencies()
+
+        frecencies[chosen_character] = frecencies.get(chosen_character, 0) + 1.1
+
+        with new_file_name.open('w+') as new_file:
+            for (character, frecency) in sorted(frecencies.items(), key=lambda item: item[1], reverse=True):
+                new_file.write(f'{math.floor(frecency)} {character}\n')
+
+        new_file_name.rename(frecency_file_location)
+
     def append_to_favorites_file(self, characters: str) -> None:
         favorites_file_location.parent.mkdir(parents=True, exist_ok=True)
         with favorites_file_location.open('a+') as file:
